#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define USE_CONSOLE
#include <allegro.h>

typedef short int WORD;
typedef unsigned char UBYTE;


#define dt1_head_size 276
#define block_size     96
#define sub_tile_size  20


typedef struct
{
   WORD xpos;
   WORD ypos;
   int  xgrid;
   int  ygrid;
   WORD format;
   long data_length;
   long data_offset;
} SUB_TILE_S;

typedef struct
{
   long  direction;
   WORD  roof_y;
   UBYTE tile_sound;
   UBYTE animated;
   long  ysize;
   long  xsize;
   long  orientation;
   long  main_index;
   long  sub_index;
   long  frame;
   long  unknown;
   int   sub_tiles_flags[25];
   long  tiles_ptr;
   long  tiles_length;
   long  tiles_number;
} BLOCK_S;


typedef struct
{
   long block;
   char pcx_file[20];
   int  pcx_x;
   int  pcx_y;
   long direction;
   int  roof_y;
   int  tile_sound;
   int  animated;
   long orientation;
   long main_index;
   long sub_index;
   long frame_rarity;
   long unknown;
   int  sub_walk[25];
} TXT_LINE_DATA_S;


// global datas
typedef         enum {PT_FA, PT_FS, PT_WU, PT_WD, PT_MAX} PCX_TYPE;
FILE            * ini = NULL, * dt1 = NULL;
BITMAP          * pcx[4] = {NULL, NULL, NULL, NULL};
PALETTE         the_pal;
BLOCK_S         block;
SUB_TILE_S      sub_tile;

#define TXT_LINE_MAX 32000

TXT_LINE_DATA_S txt_line_data[TXT_LINE_MAX];

long            block_number = 0, ini_line_num = 1;


// ==========================================================================
// atexit() function, when the program end, to close the files
void dt1make_exit(void)
{
   int i;
   
   if (ini != NULL) fclose(ini);
   if (dt1 != NULL) fclose(dt1);
   for (i=0; i<PT_MAX; i++)
      if (pcx[i] != NULL)
         destroy_bitmap(pcx[i]);
}


// ==========================================================================
// write the dt1 main header
void write_dt1_header(void)
{
   long x1 = 7, x2 = 6, head_start = dt1_head_size;
   int  i;

   // signature
   fwrite(&x1, 1, 4, dt1);
   fwrite(&x2, 1, 4, dt1);
   
   // zeros
   for (i=0; i<260; i++)
      fputc(0, dt1);
   
   // # of blocks
   fwrite(&block_number, 1, 4, dt1);

   // start of block header, always 0x114
   fwrite(&head_start, 1, 4, dt1);
}


// ==========================================================================
// write fake blocks headers
// will be set later
void write_fake_blocks_headers(void)
{
   int i, b;

   for (b=0; b<block_number; b++)
      for (i=0; i<block_size; i++)
         fputc(0, dt1);
}


// ==========================================================================
// write / update 1 block header
void write_block_header(long b)
{
   long old_pos, new_pos;
   int  i;

   old_pos = ftell(dt1);
   new_pos = dt1_head_size + (block_size * b);
   fseek(dt1, new_pos, SEEK_SET);
   
   // direction
   fwrite(&block.direction, 1, 4, dt1);

   // flags
   fwrite(&block.roof_y,     1, 2, dt1);
   fwrite(&block.tile_sound, 1, 1, dt1);
   fwrite(&block.animated,   1, 1, dt1);
   
   // ysize and xsize. ysize must be negative
   if (block.ysize > 0)
      block.ysize = - block.ysize;
   fwrite(&block.ysize, 1, 4, dt1);
   fwrite(&block.xsize, 1, 4, dt1);

   // zeros
   for (i=0; i<4; i++)
      fputc(0, dt1);
   
   // seems to be that
   fwrite(&block.orientation, 1, 4, dt1);
   fwrite(&block.main_index,  1, 4, dt1);
   fwrite(&block.sub_index,   1, 4, dt1);
   fwrite(&block.frame,       1, 4, dt1);

   // unknown (like 0xFF00FF00)
   fwrite(&block.unknown, 1, 4, dt1);

   // sub-tiles flags
   for (i=0; i<25; i++)
       fputc(block.sub_tiles_flags[i], dt1);

   // zeros
   for (i=0; i<7; i++)
      fputc(0, dt1);
      
   // pointer to sub-tiles headers
   fwrite(&block.tiles_ptr,    1, 4, dt1);

   // sub-tiles length
   fwrite(&block.tiles_length, 1, 4, dt1);

   // # of sub-tiles
   fwrite(&block.tiles_number, 1, 4, dt1);

   // zeros
   for (i=0; i<12; i++)
      fputc(0, dt1);

   // end, get back
   fseek(dt1, old_pos, SEEK_SET);
}


// ==========================================================================
// write fake sub-tiles headers of 1 block
// will be set later
void write_fake_sub_tiles_headers(long n)
{
   long i, t;

   for (t=0; t<n; t++)
      for (i=0; i<sub_tile_size; i++)
         fputc(0, dt1);
}


// ==========================================================================
// update the sub-tile header of 1 block
void update_sub_tile_header(void)
{
   int i;
   
   // xpos & ypos
   fwrite(&sub_tile.xpos, 1, 2, dt1);
   fwrite(&sub_tile.ypos, 1, 2, dt1);

   // zero
   for (i=0; i<2; i++)
      fputc(0, dt1);

   // xgrid & ygrid
   fputc(sub_tile.xgrid, dt1);
   fputc(sub_tile.ygrid, dt1);

   // format
   fwrite(&sub_tile.format, 1, 2, dt1);

   // data length
   fwrite(&sub_tile.data_length, 1, 4, dt1);

   // zero
   for (i=0; i<2; i++)
      fputc(0, dt1);

   // data offset
   fwrite(&sub_tile.data_offset, 1, 4, dt1);
}


// ==========================================================================
// check if it can be a 3d-isometric
// if at least 1 trans pixel then it's a trans isometric
int is_solid_isometric(BITMAP * src, int x0, int y0)
{
   int   x, y=0, n,
         xjump[15] = {14, 12, 10, 8, 6, 4, 2, 0, 2, 4, 6, 8, 10, 12, 14},
         nbpix[15] = {4, 8, 12, 16, 20, 24, 28, 32, 28, 24, 20, 16, 12, 8, 4},
         length=256;

   while (length > 0)
   {
      x = xjump[y];
      n = nbpix[y];
      length -= n;
      while (n)
      {
         if (getpixel(src, x0+x, y0+y) == 0)
            return FALSE;
         x++;
         n--;
      }
      y++;
   }
   return TRUE;
}


// ==========================================================================
// check if it is an empty 3d-isometric tile
// if at least 1 solid pixel then it's a non-empty isometric
int is_all_empty_isometric(BITMAP * src, int x0, int y0)
{
   int   x, y=0, n,
         xjump[15] = {14, 12, 10, 8, 6, 4, 2, 0, 2, 4, 6, 8, 10, 12, 14},
         nbpix[15] = {4, 8, 12, 16, 20, 24, 28, 32, 28, 24, 20, 16, 12, 8, 4},
         length=256;

   while (length > 0)
   {
      x = xjump[y];
      n = nbpix[y];
      length -= n;
      while (n)
      {
         if (getpixel(src, x0+x, y0+y) != 0)
            return FALSE;
         x++;
         n--;
      }
      y++;
   }
   return TRUE;
}


// ==========================================================================
// check if it is an empty tile (rectangular)
// if at least 1 solid pixel then it's a non-empty sub-tile of wall
// btw, wall's sub-tiles are always coded in the format with transparency
int is_all_empty_wall(BITMAP * src, int x0, int y0)
{
   int x, y;

   for (y=0; y<32; y++)
      for (x=0; x<32; x++)
         if (getpixel(src, x0+x, y0+y) != 0)
            return FALSE;
   return TRUE;
}


// ==========================================================================
// write a 3d-isometric sub-tile datas
void write_isometric(BITMAP * src, int x0, int y0)
{
   int   x, y=0, n,
         xjump[15] = {14, 12, 10, 8, 6, 4, 2, 0, 2, 4, 6, 8, 10, 12, 14},
         nbpix[15] = {4, 8, 12, 16, 20, 24, 28, 32, 28, 24, 20, 16, 12, 8, 4},
         length=256;

   while (length > 0)
   {
      x = xjump[y];
      n = nbpix[y];
      length -= n;
      while (n)
      {
         fputc(getpixel(src, x0+x, y0+y), dt1);
         x++;
         n--;
      }
      y++;
   }
}


// ==========================================================================
// write sub-tile datas with transparency
// pass TRUE to is_3diso if it is a transparent FLOOR
void write_tile_with_transparency(BITMAP * src, int x0, int y0, int is_3diso)
{
   BITMAP * tmp;
   int    x, y=0, n, c,
          xjump[15] = {14, 12, 10, 8, 6, 4, 2, 0, 2, 4, 6, 8, 10, 12, 14},
          nbpix[15] = {4, 8, 12, 16, 20, 24, 28, 32, 28, 24, 20, 16, 12, 8, 4},
          length=256,
          nb_trans, nb_solid, max_y;

   tmp = create_bitmap(32, 32);
   if (tmp == NULL)
      return;
   clear(tmp);
   if (is_3diso == TRUE)
   {
      max_y = 15;
      while (length > 0)
      {
         x = xjump[y];
         n = nbpix[y];
         length -= n;
         while (n)
         {
            c = getpixel(src, x0+x, y0+y);
            putpixel(tmp, x, y, c);
            x++;
            n--;
         }
         y++;
      }
   }
   else
   {
      max_y = 32;
      blit(src, tmp, x0, y0, 0, 0, 32, 32);
   }


   // encode
   x = y = 0;
   while (y < max_y)
   {
      // nb_trans
      c = getpixel(tmp, x, y);
      nb_trans = 0;
      if (c == 0)
      {
         x++;
         nb_trans = 1;
         while ( (x < 32) && (getpixel(tmp, x, y) == 0) )
         {
            x++;
            nb_trans++;
         }
      }
      if (x < 32)
      {
         fputc(nb_trans, dt1);
         // nb_solid
         nb_solid = 1;
         x++;
         while ( (x < 32) && (getpixel(tmp, x, y) != 0 ) )
         {
            x++;
            nb_solid++;
         }
         fputc(nb_solid, dt1);
         x -= nb_solid;
         while (nb_solid)
         {
            fputc(getpixel(tmp, x, y), dt1);
            x++;
            nb_solid--;
         }
      }
      // EOL
      if (x >= 32)
      {
         fputc(0, dt1);
         fputc(0, dt1);
         x = 0;
         y++;
      }
   }

   // end
   destroy_bitmap(tmp);
   return;
}


// ==========================================================================
// write each sub-tile of a floor
void write_floor_loop(BITMAP * src, int x0, int y0)
{
   struct
   {
      int x;
      int y;
   } pos[25] = { { 64, 64}, { 48, 56}, {32, 48}, {16, 40}, { 0, 32},
                 { 80, 56}, { 64, 48}, {48, 40}, {32, 32}, {16, 24},
                 { 96, 48}, { 80, 40}, {64, 32}, {48, 24}, {32, 16},
                 {112, 40}, { 96, 32}, {80, 24}, {64, 16}, {48,  8},
                 {128, 32}, {112, 24}, {96, 16}, {80,  8}, {64,  0}
               };
   long x, y, n=0, start_header, start_data, end_data;
   int  i, present[25];

   // for each sub-tiles, count the present ones
   for (i=0; i<25; i++)
      present[i] = FALSE;
      
   for (i=0; i<25; i++)
   {
      x = pos[i].x;
      y = pos[i].y;
      if (is_all_empty_isometric(src, x0+x, y0+y) == FALSE)
      {
         present[i] = TRUE;
         n++;
      }
   }

   // write sub-tiles headers
   end_data = start_header = ftell(dt1);
   write_fake_sub_tiles_headers(n);

   // write sub-tiles datas
   n=0;
   for (i=0; i<25; i++)
   {
      if (present[i])
      {
         start_data = ftell(dt1);
         x = pos[i].x;
         y = pos[i].y;
         if (is_solid_isometric(src, x0+x, y0+y))
         {
            write_isometric(src, x0+x, y0+y);
            sub_tile.format = 0x0001;
         }
         else
         {
            write_tile_with_transparency(src, x0+x, y0+y, TRUE);
            sub_tile.format = 0x2005;
         }
         // sub-tile header update
         end_data = ftell(dt1);
         sub_tile.xpos = pos[i].x;
         sub_tile.ypos = pos[i].y;
         sub_tile.xgrid = 4 - (i % 5);
         sub_tile.ygrid = 4 - (i / 5);
         sub_tile.data_length = end_data - start_data;
         sub_tile.data_offset = start_data - start_header;
         fseek(dt1, start_header + (n * sub_tile_size), SEEK_SET);
         update_sub_tile_header();
         fseek(dt1, end_data, SEEK_SET);
         // next
         n++;
      }
   }

   // prepare block update
   block.ysize        = -128;
   block.xsize        = 160;
   block.tiles_ptr    = start_header;
   block.tiles_length = end_data - start_header;
   block.tiles_number = n;
}


// ==========================================================================
// write each sub-tile of a floor with walls up
void write_walls_up_loop(BITMAP * src, int x0, int y0)
{
   long x, y, n=0, start_header, start_data, end_data;
   int  nx=0, ny=50, present[50][5], nb_pres=0;

   // for each sub-tiles, count the present ones
   for (y=0; y<50; y++)
      for (x=0; x<5; x++)
         present[y][x] = FALSE;
         
   for (y=0; y<src->h; y += 32)
   {
      for (x=128; x>=0; x -= 32)
      {
         if (is_all_empty_wall(src, x0+x, y0+y) == FALSE)
         {
            present[y/32][x/32] = TRUE;
            n++;
         }
      }
   }
   // write sub-tiles headers
   end_data = start_header = ftell(dt1);
   write_fake_sub_tiles_headers(n);

   // write sub-tiles datas
   n = 0;
   for (y=0; y<src->h; y += 32)
   {
      for (x=128; x>=0; x -= 32)
      {
         if (present[y/32][x/32] == TRUE)
         {
            start_data = ftell(dt1);
            write_tile_with_transparency(src, x0+x, y0+y, FALSE);
            sub_tile.format = 0x1001;
            // sub-tile header update
            end_data = ftell(dt1);
            sub_tile.xpos = x;
            sub_tile.ypos = - src->h + y;
            sub_tile.xgrid = 0;
            sub_tile.ygrid = 0;
            sub_tile.data_length = end_data - start_data;
            sub_tile.data_offset = start_data - start_header;
            fseek(dt1, start_header + (n * sub_tile_size), SEEK_SET);
            update_sub_tile_header();
            fseek(dt1, end_data, SEEK_SET);
            // next
            n++;
         }
      }
   }
   // prepare block update
   for (y=0; y<50; y++)
   {
      for (x=0; x<5; x++)
      {
         if (present[y][x] == TRUE)
         {
            if (x>nx)
               nx = x;
            if (y<ny)
               ny = y;
            nb_pres++;
         }
      }
   }
   if (nb_pres)
   {
      nx++;
      ny = (src->h/32) - ny + 1;
      block.ysize = - (ny * 32);
      block.xsize = nx * 32;
   }
   else
   {
      block.ysize = 0;
      block.xsize = 0;
   }
   block.tiles_ptr    = start_header;
   block.tiles_length = end_data - start_header;
   block.tiles_number = n;
}


// ==========================================================================
// write each sub-tile of a floor with walls down
void write_walls_down_loop(BITMAP * src, int x0, int y0)
{
   long x, y, n=0, start_header, start_data, end_data;
   int  nx=0, ny=0, present[50][5], nb_pres=0;

   // for each sub-tiles, count the present ones
   for (y=0; y<50; y++)
      for (x=0; x<5; x++)
         present[y][x] = FALSE;
         
   for (y=0; y<src->h; y += 32)
   {
      for (x=128; x>=0; x -= 32)
      {
         if (is_all_empty_wall(src, x0+x, y0+y) == FALSE)
         {
            present[y/32][x/32] = TRUE;
            n++;
         }
      }
   }

   // write sub-tiles headers
   end_data = start_header = ftell(dt1);
   write_fake_sub_tiles_headers(n);

   // write sub-tiles datas
   n = 0;
   for (y=src->h - 32; y>=0; y -= 32)
   {
      for (x=128; x>=0; x -= 32)
      {
         if (present[y/32][x/32] == TRUE)
         {
            start_data = ftell(dt1);
            write_tile_with_transparency(src, x0+x, y0+y, FALSE);
            sub_tile.format = 0x1001;

            // sub-tile header update
            end_data = ftell(dt1);
            sub_tile.xpos = x;
            sub_tile.ypos = y - 96;
            sub_tile.xgrid = 0;
            sub_tile.ygrid = 0;
            sub_tile.data_length = end_data - start_data;
            sub_tile.data_offset = start_data - start_header;
            fseek(dt1, start_header + (n * sub_tile_size), SEEK_SET);
            update_sub_tile_header();
            fseek(dt1, end_data, SEEK_SET);
            // next
            n++;
         }
      }
   }

   // prepare block update
   for (y=0; y<50; y++)
   {
      for (x=0; x<5; x++)
      {
         if (present[y][x] == TRUE)
         {
            if (x>nx)
               nx = x;
            if (y>ny)
               ny = y;
            nb_pres++;
         }
      }
   }
   if (nb_pres)
   {
      nx++;
      ny = ny + 1; // + 2 ?
      block.ysize = - (ny * 32);
      block.xsize = nx * 32;
   }
   else
   {
      block.ysize = 0;
      block.xsize = 0;
   }

   // hack for some few cliff (block 70 in expansion\siege\cliff.dt1)
   nb_pres=0;
   for(y=0; y<4; y++)
      for (x=0; x<5; x++)
         if (present[y][x] == TRUE)
            nb_pres++;
   if (nb_pres == 0)
      block.ysize += (4 * 32);
   
   // continue
   block.tiles_ptr    = start_header;
   block.tiles_length = end_data - start_header;
   block.tiles_number = n;
}


// ==========================================================================
void ini_error(char * str)
{
   printf("error in the .ini file\n");
   printf("%s, at line %li\n", str, ini_line_num);
fflush(stdout);
   exit(1);
}


// ==========================================================================
// read a line from the txt file
int read_txt_line(FILE * in, int line_num)
{
   int ret;


   if (line_num >= TXT_LINE_MAX)
//   if (line_num >= 5)
   {
      printf("can't read line %i: max is %i, need recompilation\n",
         line_num + 1, TXT_LINE_MAX);
      return EOF;
   }

   ret = fscanf(in,
      "%li"
      "\t%s"
      "\t%i"
      "\t%i"
      "\t%li"
      "\t%i"
      "\t%i"
      "\t%i"
      "\t%li"
      "\t%li"
      "\t%li"
      "\t%li"
      "\t0x%08lX"
      "\t%i\t%i\t%i\t%i\t%i"
      "\t%i\t%i\t%i\t%i\t%i"
      "\t%i\t%i\t%i\t%i\t%i"
      "\t%i\t%i\t%i\t%i\t%i"
      "\t%i\t%i\t%i\t%i\t%i",
      & txt_line_data[line_num].block,
      & txt_line_data[line_num].pcx_file[0],
      & txt_line_data[line_num].pcx_x,
      & txt_line_data[line_num].pcx_y,
      & txt_line_data[line_num].direction,
      & txt_line_data[line_num].roof_y,
      & txt_line_data[line_num].tile_sound,
      & txt_line_data[line_num].animated,
      & txt_line_data[line_num].orientation,
      & txt_line_data[line_num].main_index,
      & txt_line_data[line_num].sub_index,
      & txt_line_data[line_num].frame_rarity,
      & txt_line_data[line_num].unknown,
      & txt_line_data[line_num].sub_walk[20],
      & txt_line_data[line_num].sub_walk[21],
      & txt_line_data[line_num].sub_walk[22],
      & txt_line_data[line_num].sub_walk[23],
      & txt_line_data[line_num].sub_walk[24],
      & txt_line_data[line_num].sub_walk[15],
      & txt_line_data[line_num].sub_walk[16],
      & txt_line_data[line_num].sub_walk[17],
      & txt_line_data[line_num].sub_walk[18],
      & txt_line_data[line_num].sub_walk[19],
      & txt_line_data[line_num].sub_walk[10],
      & txt_line_data[line_num].sub_walk[11],
      & txt_line_data[line_num].sub_walk[12],
      & txt_line_data[line_num].sub_walk[13],
      & txt_line_data[line_num].sub_walk[14],
      & txt_line_data[line_num].sub_walk[5],
      & txt_line_data[line_num].sub_walk[6],
      & txt_line_data[line_num].sub_walk[7],
      & txt_line_data[line_num].sub_walk[8],
      & txt_line_data[line_num].sub_walk[9],
      & txt_line_data[line_num].sub_walk[0],
      & txt_line_data[line_num].sub_walk[1],
      & txt_line_data[line_num].sub_walk[2],
      & txt_line_data[line_num].sub_walk[3],
      & txt_line_data[line_num].sub_walk[4]
   );

   return ret;
}



// ==========================================================================
// read a HEADERR line from the txt file
int read_txt_line_head(FILE * in, int line_num)
{
   int ret;
   char buffer[80];

   if (line_num >= 5)
   {
      printf("can't read line %i: max is %i, need recompilation\n",
         line_num + 1, TXT_LINE_MAX);
      return EOF;
   }

   ret = fscanf(in,
      "%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s"
      "\t%s\t%s\t%s\t%s\t%s"
      "\t%s\t%s\t%s\t%s\t%s"
      "\t%s\t%s\t%s\t%s\t%s"
      "\t%s\t%s\t%s\t%s\t%s"
      "\t%s\t%s\t%s\t%s\t%s",
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer,
      buffer
   );

   return ret;
}


// ==========================================================================
void read_txt_all(char * txt_name)
{
   FILE * in;
   int  line_num, i;
   long pos;

   char error[256];
   long b = 0, cur_b = 0, pcx_x, pcx_y;
   int  pcx_idx;


   in = fopen(txt_name, "rt");
   if (in == NULL)
   {
      printf("can't read %s\n", txt_name);
      return;
   }

   // skip header line
   if (read_txt_line_head(in, 0) == EOF)
   {
      printf("closing %s\n", txt_name);
      fclose(in);
      return;
   }

   // count number of blocks
   pos = ftell(in);
   line_num = 0;
   while (read_txt_line(in, line_num) != EOF)
   {
      // next line
      line_num++;
   }
   block_number = line_num;
   fseek(in, pos, SEEK_SET);
   write_dt1_header();
   write_fake_blocks_headers();

   // read entire .txt
   line_num = 0;
   b        = 0;
   while (read_txt_line(in, line_num) != EOF)
   {
      memset(&block, 0, sizeof(block));
      if (cur_b != txt_line_data[line_num].block)
      {
         sprintf(error, "block number is not what it ought to be\n(%li instead of %li)",
            cur_b, txt_line_data[line_num].block);
         ini_error(error);
      }

      if (strcmp(txt_line_data[line_num].pcx_file, "fa") == 0)
         pcx_idx = PT_FA;
      else if (strcmp(txt_line_data[line_num].pcx_file, "fs") == 0)
         pcx_idx = PT_FS;
      else if (strcmp(txt_line_data[line_num].pcx_file, "wu") == 0)
         pcx_idx = PT_WU;
      else if (strcmp(txt_line_data[line_num].pcx_file, "wd") == 0)
         pcx_idx = PT_WD;
      else
      {
         sprintf(error, "unknown pcx code (%s)", txt_line_data[line_num].pcx_file);
         ini_error(error);
      }

      pcx_x             = txt_line_data[line_num].pcx_x;
      pcx_y             = txt_line_data[line_num].pcx_y;
      block.direction   = txt_line_data[line_num].direction;
      block.roof_y      = txt_line_data[line_num].roof_y;
      block.tile_sound  = txt_line_data[line_num].tile_sound;
      block.animated    = txt_line_data[line_num].animated;
      block.orientation = txt_line_data[line_num].orientation;
      block.main_index  = txt_line_data[line_num].main_index;
      block.sub_index   = txt_line_data[line_num].sub_index;
      block.frame       = txt_line_data[line_num].frame_rarity;
      block.unknown     = txt_line_data[line_num].unknown;
      for (i=0; i < 25; i++)
         block.sub_tiles_flags[i] = txt_line_data[line_num].sub_walk[i];

      switch(pcx_idx)
      {
         case PT_FA :
         case PT_FS :
            write_floor_loop(pcx[pcx_idx], pcx_x, pcx_y);
            break;
            
         case PT_WU :
            write_walls_up_loop(pcx[PT_WU], pcx_x, pcx_y);
            break;

         case PT_WD :
            write_walls_down_loop(pcx[PT_WD], pcx_x, pcx_y);
            break;
      }
      write_block_header(b);
      cur_b++;
      b++;

      // next line
      line_num++;
   }

   printf("%i lines were read in %s\n", line_num, txt_name);

   // end
   fclose(in);
}


// ==========================================================================
int main(int argc, char * argv[])
{
   char ini_path[256], ini_name[256], ini_ext[256], tmp_file[256];
   int  i, drive_pos = -1, last_slash_pos = -1, period_pos = -1, n;
   
   // header & syntaxe & atexit()
   printf("DT1MAKE2 v 1.00, by Paul Siramy, Freeware\n"
          "=========================================\n");

   if (argc == 1)
   {
      printf("syntaxe : dt1make <file.ini>\n");
      exit(0);
   }
/*
   ini = fopen(argv[1], "rt");
   if (ini == NULL)
   {
      printf("%s not found\n", argv[1]);
      exit(1);
   }
*/   

   // install allegro
   allegro_init();
   set_color_depth(8);
   atexit(dt1make_exit);

   // pcx filenames managment
   period_pos = n = strlen(argv[1]);
   for (i=0; i<n; i++)
   {
      if ( argv[1][i] == ':')
         drive_pos = i;
      else if ( (argv[1][i] == '\\') || (argv[1][i] == '/') )
         last_slash_pos = i;
      if ( argv[1][i] == '.')
         period_pos = i;
   }
   memset(ini_path, 0, sizeof (ini_path));
   memset(ini_name, 0, sizeof (ini_name));
   memset(ini_ext,  0, sizeof (ini_ext));
   i = 1 + (last_slash_pos > drive_pos ? last_slash_pos : drive_pos);
   strncpy(ini_path, argv[1], i);
   strncpy(ini_name, argv[1] + i, period_pos - i);
   strncpy(ini_ext,  argv[1] + period_pos, n - period_pos);
   
   // open pcx
   sprintf(tmp_file, "%s%s-fa.pcx", ini_path, ini_name);
// fprintf(stderr, "loading %s\n", tmp_file);
// fflush(stderr);
   pcx[PT_FA] = load_pcx(tmp_file, the_pal);

   sprintf(tmp_file, "%s%s-fs.pcx", ini_path, ini_name);
// fprintf(stderr, "loading %s\n", tmp_file);
// fflush(stderr);
   pcx[PT_FS] = load_pcx(tmp_file, the_pal);

   sprintf(tmp_file, "%s%s-wu.pcx", ini_path, ini_name);
// fprintf(stderr, "loading %s\n", tmp_file);
// fflush(stderr);
   pcx[PT_WU] = load_pcx(tmp_file, the_pal);

   sprintf(tmp_file, "%s%s-wd.pcx", ini_path, ini_name);
// fprintf(stderr, "loading %s\n", tmp_file);
// fflush(stderr);
   pcx[PT_WD] = load_pcx(tmp_file, the_pal);

   // dt1
   sprintf(tmp_file, "NEW_%s.dt1", ini_name);
   dt1 = fopen(tmp_file, "wb+");
   
   // read ini
   read_txt_all(argv[1]);
   printf("\n\"%s\" has been saved\n", tmp_file);
   
   // end
   printf("done\n");
   return 0;
}
END_OF_MAIN();

