#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define USE_CONSOLE
#include <allegro.h>

int main(int argc, char ** argv)
{
   BITMAP  * bmp;
   PALETTE pal;
   FILE    * in;
   int     cmap[256], finalcmap[256], i, x, y, c;
   int     r[256], g[256], b[256], col;
   unsigned long dr, dg, db, dist, min_dist;


   if (argc != 3)
   {
      printf("syntaxe : %s <image.pcx> <palette.dat>\n"
             "\n"
             "   Note : <palette.dat> is a Diablo II palette\n",
             argv[0]
      );
      return 1;
   }

   allegro_init();
   bmp = load_pcx(argv[1], pal);
   if (bmp == NULL)
   {
      printf("can't open %s\n", argv[1]);
      return 2;
   }
   if (bitmap_color_depth(bmp) != 8)
   {
      destroy_bitmap(bmp);
      printf("%s is not a 256 color image\n", argv[1]);
      return 3;
   }

   // cmap
   in = fopen("colormap.txt", "rt");
   if (in == NULL)
   {
      destroy_bitmap(bmp);
      printf("can't open colormap.txt\n");
      return 4;
   }
   for (i=0; i < 256; i++)
      fscanf(in, "%i", & cmap[i]);
   fclose(in);

   // d2 palette
   in = fopen(argv[2], "rb");
   if (in == NULL)
   {
      destroy_bitmap(bmp);
      printf("can't open %s\n", argv[2]);
      return 5;
   }
   for (i=0; i < 256; i++)
   {
      b[i] = fgetc(in);
      g[i] = fgetc(in);
      r[i] = fgetc(in);
   }
   fclose(in);

   // -1 values
   cmap[0] = 0;
   finalcmap[0] = 0;
   for (i=1; i < 256; i++)
   {
      c = cmap[i];
      finalcmap[i] = c < 0 ? -c : c;
      if (c == -1)
      {
         // find new best color
         col = 0;
         min_dist = 0x7FFFFFFFLU;
         for (x=1; x < 256; x++)
         {
            if (cmap[x] >= 0)
            {
               // color NOT forbiden
               dr = r[i] - r[x];
               dr = dr * dr * 76;

               dg = g[i] - g[x];
               dg = dg * dg * 150;

               db = b[i] - b[x];
               db = db * db * 28;

               dist = dr + dg + db;

               if (dist < min_dist)
               {
                  min_dist = dist;
                  col = x;
               }
            }
         }
         finalcmap[i] = col;
      }
   }

   // debug
/*
   for (i=0; i < 256; i++)
   {
      c = finalcmap[i];
      printf("cmap[%3i] (%3i %3i %3i) ---> %3i (%3i %3i %3i)\n",
         i,
         r[i], g[i], b[i],
         c,
         r[c], g[c], b[c]         
      );
   }
*/
   // remap pixels
   // fprintf(stderr, "processing %s\n", argv[1]);
   // fflush(stderr);

   for (y=0; y < bmp->h; y++)
   {
      for (x=0; x < bmp->w; x++)
      {
         c = bmp->line[y][x];
         bmp->line[y][x] = finalcmap[c];
      }
   }

   // save back image
   save_pcx(argv[1], bmp, pal);

   // end
   destroy_bitmap(bmp);
   return 0;
}
END_OF_MAIN();
